#!/bin/bash

# 20200613 fredx181, code_tray, run selected code snippet (e.g. select in browser) in xterm, requires yad and xclip
# partly taken from codes shared in 'YAD - Tips' thread, thanks specially misko, vovchik, mochimoppel, stemsee 
# left-click the icon to run the selected code
# 20200617 fredx181, added to menu 'Save directory' to choose directory for saved codes

export FPROG="$0"
export PROG=$(basename "$0")
STARTUP=$(readlink -f "$HOME/Startup/$PROG")
if [ "$FPROG" = "$HOME/Startup/$PROG" ]; then
FPROG="$STARTUP"
fi
running=`ps -e | grep -w "$PROG" | wc -l`
[[ "$running" -gt 2 ]] && exit  # run only one instance

if [ -z $(busybox which gxmessage) ]; then
MESSAGE=xmessage
else
MESSAGE=gxmessage
fi

if [ -z `busybox which yad` ]; then
	msg=" You don't have yad installed.\nIt's a dependency of this program.\n Please install it."
	echo $msg
	$MESSAGE "`echo -e $msg`"
exit 0
fi

if [ -z `busybox which xclip` ]; then
	msg=" You don't have xclip installed.\nIt's a dependency of this program.\n Please install it."
	echo $msg
	$MESSAGE "`echo -e $msg`"
exit 0
fi

VERSION=`yad --version | awk '{ print $1 }'`
verlte() {
    [  "$1" = "`echo -e "$1\n$2" | sort -V | head -n1`" ]
}

verlt() {
    [ "$1" = "$2" ] && return 1 || verlte $1 $2
}

if verlt $VERSION 0.37.0; then
yad --text=" The version of yad installed is too old for to run this program,  \n Please upgrade yad to a version higher than 0.36   " --button="gtk-close"
exit
fi

mkdir -p $HOME/.code_tray/geometry
mkdir -p $HOME/.code_tray/saved
mkdir -p /tmp/code_tray/

 # create preference file if not exists
if [ ! -f $HOME/.code_tray/prefs ]; then
echo 'CONF_EXIT=TRUE
ON_TOP=FALSE
SHOW_ORG=FALSE
FONT="Monospace Bold 11"
NBSP_TO_SPACE=FALSE
CD="$HOME"
_TERM="xterm -rv -geometry 80x15"
TEXTEDITOR="internal editor"
' > $HOME/.code_tray/prefs
fi

# for if $HOME/.code_tray/prefs already exists, adding SDIR= (save directory)
. $HOME/.code_tray/prefs
  if [ "$SDIR" = "" ]; then
echo "SDIR='$HOME/.code_tray/saved'" >> $HOME/.code_tray/prefs
  fi
. $HOME/.code_tray/prefs
export SDIR="$SDIR"
export CD="$CD"

export ICON="$HOME/.code_tray/x_term.svg"
if [ ! -e "$ICON" ]; then
	SVG='<svg xmlns:rdf="http://www.w3.org/1999/02/22-rdf-syntax-ns#" xmlns="http://www.w3.org/2000/svg" height="24" width="24" version="1.1" xmlns:cc="http://creativecommons.org/ns#" xmlns:dc="http://purl.org/dc/elements/1.1/">
 <g transform="translate(0 -1028.4)">
  <path d="m3 1030.4c-1.1046 0-2 0.9-2 2v7 2 7c0 1.1 0.8954 2 2 2h9 9c1.105 0 2-0.9 2-2v-7-2-7c0-1.1-0.895-2-2-2h-9-9z" fill="#2c3e50"/>
  <path d="m3 2c-1.1046 0-2 0.8954-2 2v3 3 1 1 1 3 3c0 1.105 0.8954 2 2 2h9 9c1.105 0 2-0.895 2-2v-3-4-2-3-3c0-1.1046-0.895-2-2-2h-9-9z" transform="translate(0 1028.4)" fill="#34495e"/>
  <path d="m4 5.125v1.125l3 1.75-3 1.75v1.125l5-2.875-5-2.875zm5 4.875v1h5v-1h-5z" transform="translate(0 1028.4)" fill="#ecf0f1"/>
 </g>
</svg>'
	echo "$SVG" > $HOME/.code_tray/x_term.svg
fi

#[ -n "$(which xterm)" ] && export _TERM="xterm -rv -geometry 80x15" || export _TERM="rxvt -rv -geometry 80x15"

s_dir () {
. $HOME/.code_tray/prefs
S_DIR="$1"
cd "$S_DIR"
S_DIR=$(yad --file --directory --width 600 --height 500 --title="Save directory" --text="Select directory for saved codes")
[ $? -ne 0 ] && exit

sed -i "s|^SDIR=.*|SDIR='$S_DIR'|" $HOME/.code_tray/prefs


export SDIR="$S_DIR"
echo menu:"Show selected code!bash -c code_paste!gtk-edit|View saved codes!bash -c savedtexts!gtk-save|Save directory, currently: $SDIR!bash -c 's_dir $SDIR'!gtk-directory|Change run directory, currently: $DIR!bash -c 'change_dir $DIR'!gtk-directory|Preferences!bash -c code_settings!gtk-preferences|Help!bash -c usage_info!gtk-info|Quit!bash -c quit_tray!gtk-quit" >$PIPE
}; export -f s_dir
 

usage_info () {
TEXT="<b>*** Code-tray usage help ***</b>

- Select some code (bash) (no need to copy, just select it), e.g. in browser or text file.
Then by left-clicking the icon, a terminal will appear running the code.

- From menu (right-click) there are some other options, e.g:
    - Preferences, to change some default settings, e.g. texteditor, terminal, set font
    - Show selected code, save to a file if you wish 
   (can be handy to edit it, select it, then run it by left-clicking the icon)
    - View saved codes (located in ~/.code_tray/saved)
    - Save directory, choose where to save codes

<b>Hint:</b> If you need to edit selected code, run 'Show selected code' from menu,
then edit it, select it and run by left-clicking the icon

Required is to have 'xclip' installed.
" 
yad --mouse --borders=7 --title="Usage info" --window-icon=$ICON --width=600 --text="$(echo -e "$TEXT")" --on-top --button="Close!gtk-close:0"
}
export -f usage_info

code_settings () {
. $HOME/.code_tray/prefs 
[ "$ON_TOP" = "TRUE" ] && ONTOP="--on-top"
[ "$TEXTEDITOR" = "" ] && TEXTEDITOR="internal editor" && echo "TEXTEDITOR='internal editor'"
[ "$_TERM" = "" ] && _TERM="xterm -rv -geometry 80x15" && echo "_TERM='xterm -rv -geometry 80x15'"
[ "$FONT" = "" ] && FONTNAME="Monospace Bold 11" || FONTNAME="$FONT"
EDITOR="$TEXTEDITOR!defaulttexteditor!internal editor!geany!leafpad!mousepad!kwrite!kate!joe!pluma"
T_EDITOR="$TEXTEDITOR!$(echo "$EDITOR" | sed "s/$TEXTEDITOR!//g")"
_TERMINAL="$_TERM!xterm -rv -geometry 80x15!xfce4-terminal --disable-server --hold!sakura -h!mate-terminal"
_TERMINAL="$_TERM!$(echo "$_TERMINAL" | sed "s/$_TERM!//g")"

CHOICE=$(
( yad --plug=$$ --tabnum=1 --list \
    --text="  Make your selections     " \
    --checklist --borders 5 \
    --column="" --column="Option" --hide-column 2 --column="Name" \
    "$SHOW_ORG" "showoriginal" "Show selected code on top in terminal" \
    "$CONF_EXIT" "confexit" "Show confirmation window when Quit from Menu" \
    "$ON_TOP" "ontop" "Show text window on top of other windows" \
    "$NBSP_TO_SPACE" "nbsp" "Convert non-breaking space to regular space" &
yad --plug=$$ --tabnum=2 --form \
 --field="Texteditor to open saved codes :CBE" "$T_EDITOR" \
 --field="Select font for text-info dialogs :FN" "$FONTNAME" \
 --field="Terminal (with preferred options):\n(e.g. -geometry): ":CBE "$_TERMINAL" &

yad --title="Settings" --mouse --window-icon=gtk-preferences --width=530 --height=360 --paned --splitter=175 --key=$$ --orient=vert --buttons-layout=spread $ONTOP
))  
[ $? -ne 0 ] && exit
export EDITOR=$(echo "$CHOICE" | grep -v "TRUE|" | cut -d "|" -f 1)
export FONTNAME="`echo "$CHOICE" | grep -v "TRUE|" | cut -d "|" -f 2`"
export TERMCMD="`echo "$CHOICE" | grep -v "TRUE|" | cut -d "|" -f 3`"

if [ "$TERMCMD" = "" ]; then
echo "_TERM='$TERMCMD'" >> $HOME/.code_tray/prefs
else
sed -i "s/^_TERM=.*/_TERM='$TERMCMD'/" $HOME/.code_tray/prefs
fi
if [ "$FONT" = "" ]; then
echo "FONT='$FONTNAME'" >> $HOME/.code_tray/prefs
else
sed -i "s/^FONT=.*/FONT='$FONTNAME'/" $HOME/.code_tray/prefs
fi
if [ "$EDITOR" != "" ]; then
  if [ "$TEXTEDITOR" = "" ]; then
echo "TEXTEDITOR='$EDITOR'" >> $HOME/.code_tray/prefs
  else
sed -i "s/^TEXTEDITOR=.*/TEXTEDITOR='$EDITOR'/" $HOME/.code_tray/prefs
  fi
fi
if [ "$NBSP_TO_SPACE" = "" ]; then
echo "NBSP_TO_SPACE=$NBSP_TO_SPACE" >> $HOME/.code_tray/prefs
fi

if [[ "$CHOICE" =~ "TRUE|showoriginal" ]]; then
sed -i 's/^SHOW_ORG=.*/SHOW_ORG=TRUE/' $HOME/.code_tray/prefs
else
sed -i 's/^SHOW_ORG=.*/SHOW_ORG=FALSE/' $HOME/.code_tray/prefs
fi
if [[ "$CHOICE" =~ "TRUE|confexit" ]]; then
sed -i 's/^CONF_EXIT=.*/CONF_EXIT=TRUE/' $HOME/.code_tray/prefs
else
sed -i 's/^CONF_EXIT=.*/CONF_EXIT=FALSE/' $HOME/.code_tray/prefs
fi
if [[ "$CHOICE" =~ "TRUE|ontop" ]]; then
sed -i 's/^ON_TOP=.*/ON_TOP=TRUE/g' $HOME/.code_tray/prefs
else
sed -i 's/^ON_TOP=.*/ON_TOP=FALSE/g' $HOME/.code_tray/prefs
fi
if [[ "$CHOICE" =~ "TRUE|nbsp" ]]; then
sed -i 's/^NBSP_TO_SPACE=.*/NBSP_TO_SPACE=TRUE/' $HOME/.code_tray/prefs
export NBSP_TO_SPACE=TRUE
else
sed -i 's/^NBSP_TO_SPACE=.*/NBSP_TO_SPACE=FALSE/' $HOME/.code_tray/prefs
export NBSP_TO_SPACE=FALSE
fi

}; export -f code_settings

function refresh (){
. $HOME/.code_tray/prefs
      echo -e '\f' >/tmp/code_tray/PT
      echo "$(for f in $SDIR/*; do [[ -f "${f}" ]] && echo "${f##*/}"; done)" >/tmp/code_tray/PT
}; export -f refresh

function savedtexts (){
. $HOME/.code_tray/prefs
export SDIR="$SDIR"
[ -z "$FONT" ] && FONT="Monospace Bold 11"
export TEXTEDITOR="$TEXTEDITOR"
[ "$ON_TOP" = "TRUE" ] && export ONTOP="--on-top"
   rm -f /tmp/code_tray/TP
   export stxtp=/tmp/code_tray/TP
   mkfifo -m 755 $stxtp
   exec 5<> $stxtp
   rm -f /tmp/code_tray/PT
   export ptxts=/tmp/code_tray/PT
   mkfifo -m 755 $ptxts
   exec 6<> $ptxts

function delete (){
#rm -f /tmp/code_tray/fresh
rm -f $SDIR/$(cat /tmp/code_tray/sel-file)
refresh
echo -e '\f' >$stxtp
rm -f /tmp/code_tray/sel-file
}; export -f delete

function re_name (){
set -x
cd $SDIR
FILE=$(cat /tmp/code_tray/sel-file)
RENAME=$(yad --mouse --width=400 --window-icon="$ICON" --title="Rename" --entry --entry-text="$FILE" $ONTOP)
[ $? -ne 0 ] && exit
LIST="$(for f in $SDIR/*; do [[ -f "${f}" ]] && echo "${f##*/}"; done)"
[ "$FILE" = "$RENAME" ] && exit
if [ "$(echo -e "$LIST" | grep -w "$RENAME")" = "$RENAME" ]; then
yad --mouse --title="Replace ?" --borders=6 --window-icon=$ICON --text=" $RENAME already exists \n Replace it ?   " --width=400 --button="No!gtk-no:1" --button="Yes!gtk-yes:0" $ONTOP
[ $? -ne 0 ] && exit
   fi
mv "$FILE" "$RENAME"
refresh
rm -f /tmp/code_tray/sel-file
}; export -f re_name

function menu_action () {
. $HOME/.code_tray/prefs
export TEXTEDITOR="$TEXTEDITOR"
MENU_PID=$(cat /tmp/code_tray/sttmenu_pid)
  # figure out what was selected
  case "$2" in
     "edit")  kill $MENU_PID;
[ -z "$TEXTEDITOR" ] && TEXTEDITOR="internal editor"

if [ "$TEXTEDITOR" = "internal editor" ]; then
 cd "$SDIR"
             cat "$1" | yad --window-icon="$ICON" --width=700 --height=600 --title="Edit: $1" --editable --text-info --margins 3 --fontname="$FONT" --listen $ONTOP > /tmp/Code-Text-tmp
              if [ $? -eq 0 ]
              then
                  mv /tmp/Code-Text-tmp "$1"
                  rm -f /tmp/Code-Text-tmp
                  echo -e '\f' >/tmp/code_tray/TP
                  cat  "$1" >/tmp/code_tray/TP
                  echo -e '\n' >/tmp/code_tray/TP
               fi
else
cd "$SDIR"
$TEXTEDITOR "$1"
       echo -e '\f' >/tmp/code_tray/TP
       cat  "$1" >/tmp/code_tray/TP
       echo -e '\n' >/tmp/code_tray/TP
fi
               ;;
     "print") kill $MENU_PID; yad --text "Print action activated" --mouse $ONTOP;;
     "delete") kill $MENU_PID; bash -c delete;;
     "rename") kill $MENU_PID; bash -c re_name;;
     "refresh") kill $MENU_PID; bash -c refresh;;
     "run code") kill $MENU_PID; bash -c code_run;;
     "select editor") kill $MENU_PID; bash -c code_settings;;
# EDITOR=$(yad --center --window-icon="/tmp/trans_tray/all.svg" --title="Select editor" --form --field="Select or type texteditor:CBE" "internal editor!geany!leafpad!mousepad!gedit!kwrite!kate!joe!pluma" $ONTOP)
#EDITOR="`echo $EDITOR | cut -d "|" -f 1`"
#[ -z "$EDITOR" ] && exit
#echo $EDITOR > $HOME/.code_tray/text-editor;;
     "copy") kill $MENU_PID; cd "$SDIR"
      yad --window-icon="$ICON" $ONTOP --mouse --file --width=600 --height=400 --save --text="Copy to location / rename";;
   esac     
}; export -f menu_action

function menu (){

yad --list --width=200 --height=150 --mouse \
--column action --no-headers "" "edit" "rename" "delete" "select editor" \
--close-on-unfocus --on-top --no-buttons --undecorated \
--select-action="bash -c \"menu_action "$1" "%s"\"" &
 echo $! > /tmp/code_tray/sttmenu_pid
}; export -f menu

trap "rm -f $stxtp $ptxts" EXIT

yad --plug=$$ --tabnum=1 --list --text="Double click raises menu" --item-separator=" " --listen --column="Saved Codes" \
    $(for f in $SDIR/*; do [[ -f "${f}" ]] && echo "${f##*/}"; done) \
   --dclick-action="bash -c \"menu "%s"\"" \
    --select-action="bash -c \"echo -e '\f' >/tmp/code_tray/TP ; cd "$SDIR" ; echo "%s" >/tmp/code_tray/sel-file; cat  "%s" >/tmp/code_tray/TP ; echo -e '\n' >/tmp/code_tray/TP \"" <&6  &

yad --plug=$$ --tabnum=2 --editable --text-info --margins=3 --fontname="$FONT" --listen <&5 &

WINDOWTITLE="Saved-Codes"
yad --center --window-icon=gtk-save --width=750 --height=540 $ONTOP --no-buttons --paned --splitter=190 --key=$$ --title="$WINDOWTITLE" --tab="Selector" --tab="Preview" --orient=horizontal
exit
}; export -f savedtexts

# save geometry of text window
function sayonara {
   XWININFO=$(xwininfo -id $YAD_XID)
   ARRAY=(${XWININFO#* X: })
   AX=${ARRAY[0]}
   AY=${ARRAY[4]}
   RX=${ARRAY[8]}
   RY=${ARRAY[12]}
   W=${ARRAY[14]}
   H=${ARRAY[16]}
   X=$((AX-RX))
   Y=$((AY-RY))
   echo "$W"x"$H"+"$X"+"$Y" > $HOME/.code_tray/geometry/yad_code_geometry
   rm -f /tmp/code_tray/run_code
   kill -s SIGUSR1 $YAD_PID
   rm -f $PPIPE
}; export -f sayonara

code_clear () {
  echo -e '\f' >> $PPIPE
rm -f /tmp/code_tray/run_code
}
export -f code_clear

c_save () {
. $HOME/.code_tray/prefs
DATE=$(date +"%Y%m%d_%H%M%S")
   sayonara
   mv -f /tmp/code_tray/run_code_edit  $SDIR/code-$DATE
[[ -e "/tmp/code_tray/TP" && -e "/tmp/code_tray/PT" ]] && refresh
yad --title="Code saved" --borders=6 --window-icon=$ICON --text="  <b>Saved to $SDIR/code-$DATE </b>" --geometry=500x35-40-70 --undecorated --timeout 4 --no-buttons
}; export -f c_save

code_paste () {
. $HOME/.code_tray/prefs
MYTITLE="Code"
BUT="<span color='darkblue'><b>Exit</b></span>"
BUTSAVE="<span color='darkgreen'><b>Save</b></span>"
BUTCLEAR="<span color='darkred'><b>Clear</b></span>"
FONT="Monospace Bold 11"
if [ ! -e $HOME/.code_tray/geometry/yad_code_geometry ]; then
TWIDTH=$(xdpyinfo  | grep 'dimensions:' | cut -d' ' -f7 | cut -d'x' -f1)
THEIGHT=$(xdpyinfo  | grep 'dimensions:' | cut -d' ' -f7 | cut -d'x' -f2)
SWIDTH=$(( $TWIDTH - 610 ))
SHEIGHT=$(( $THEIGHT - 610 ))
#echo $SWIDTH
#echo $SHEIGHT
export GEOMETRY="600x550+$SWIDTH+$SHEIGHT"
else
export GEOMETRY="$(cat $HOME/.code_tray/geometry/yad_code_geometry 2>/dev/null)"
fi
   if [ "$ON_TOP" = "TRUE" ]; then  # on top or not
   ONTOP="--on-top"
   fi
export PPIPE="/tmp/code_tray/CODE_R"
mkfifo $PPIPE 2> /dev/null
exec 4<> $PPIPE

pyd=$(ps -eo pid,cmd | grep "yad --title=Code --window-icon=$ICON" | grep -v grep | awk '{print $1}')

if [ -z "$pyd" ]; then
yad --title="$MYTITLE" \
--window-icon="$ICON" \
--text-align=center \
--borders=6 "$ONTOP" \
--geometry="$GEOMETRY" \
--fore="black" --back="#fef1e0" \
--margins=5 \
--button="$BUTCLEAR:bash -c code_clear" \
--button="$BUTSAVE:bash -c c_save" \
--button="$BUT:bash -c sayonara" \
--text="Click <b>Save</b> for to save to a file. Click <b>Clear</b> to clear the text\nRunning again 'Show selected code' from menu, \nwill add any selected text to this window" \
--fontname="$FONT" \
--text-info --editable --listen $ONTOP > /tmp/code_tray/run_code_edit <&4 &
rm -f /tmp/code_tray/run_code
fi

if [ -z "$(xclip -o)" ]; then
echo "No code selected, please select some code and try again" >> $PPIPE
return
fi

(
echo 
if [ "$NBSP_TO_SPACE" = "TRUE" ]; then
   xclip -o | sed 's/\xc2\xa0/ /g'
else
   xclip -o
fi
echo
echo
echo --------------------------------------------
) | tee -a /tmp/code_tray/run_code >> $PPIPE
}
export -f code_paste

export PIPE="/tmp/TERM_PIPE.$RANDOM"
mkfifo $PIPE
exec 3<> $PIPE

function quit_tray() {
. $HOME/.code_tray/prefs
   if [ "$CONF_EXIT" = "TRUE" ]; then
yad --title="Exit ?" --borders=6 --window-icon=$ICON --text="   Are you sure to exit $PROG ? \n   " --button="No!gtk-no:1" --button="Yes!gtk-yes:0" --geometry=-120-90 --on-top
if [ $? -eq 0 ]; then
    echo "quit" > $PIPE
    rm -f $PIPE
    rm -f $PPIPE
fi
   else
echo "quit" >$PIPE
rm -f $PIPE
rm -f $PPIPE
   fi
}; export -f quit_tray

# add handler to manage process shutdown
function on_exit() {
    echo "quit" >$PIPE
    rm -f $PIPE
}

trap on_exit EXIT

export DIR=$CD

change_dir () {
. $HOME/.code_tray/prefs
R_DIR="$1"
cd "$R_DIR"
#DIR=$(yad --center --width=500 --title="Change Directory" --entry --entry-text="$R_DIR")
R_DIR=$(yad --file --directory --width 600 --height 500 --title="Run directory" --text="Select directory to run from")
[ $? -ne 0 ] && exit
sed -i "s|^CD=.*|CD='$R_DIR'|" $HOME/.code_tray/prefs
export DIR="$R_DIR"
echo action:'bash -c "run_code '$DIR'"' >$PIPE
echo menu:"Show selected code!bash -c code_paste!gtk-edit|View saved codes!bash -c savedtexts!gtk-save|Save directory, currently: $SDIR!bash -c 's_dir $SDIR'!gtk-directory|Change run directory, currently: $DIR!bash -c 'change_dir $DIR'!gtk-directory|Preferences!bash -c code_settings!gtk-preferences|Help!bash -c usage_info!gtk-info|Quit!bash -c quit_tray!gtk-quit" >$PIPE
}; export -f change_dir

run_code () {
. $HOME/.code_tray/prefs
DIR="$1"
cd $DIR

runc () {
eval "$(xclip -o)"
}; export -f runc 

no_sel() {
echo 'Nothing selected, please select some code to run'
echo -e '\033[1mHint:\033[0m Select any code first, then type (NOT select) \033[1m'runc'\033[0m to run it here.'
/bin/bash
#yad --center --title="Nothing selected" --text="  Nothing selected, please select some code  " --button="gtk-close:1"
}; export -f no_sel

run_sel () {
. $HOME/.code_tray/prefs
cmd=$(xclip -o)
if [ -n "$cmd" ]; then
   if [ "$SHOW_ORG" = "TRUE" ]; then
   echo -e "$cmd"
   echo -----------------------
   fi
   if [ "$NBSP_TO_SPACE" = "TRUE" ] ; then
       eval "$(xclip -o | sed 's/\xc2\xa0/ /g')"
   else
       eval "$(xclip -o)"
   fi
/bin/bash
else
no_sel
fi
}; export -f run_sel

$_TERM -e bash -c "run_sel"
[ $? -ne 0 ] && $_TERM -e "bash -c run_sel"  # some terminals won't work with above
}; export -f run_code

echo menu:"Show selected code!bash -c code_paste!gtk-edit|View saved codes!bash -c savedtexts!gtk-save|Save directory, currently: $SDIR!bash -c 's_dir $SDIR'!gtk-directory|Change run directory, currently: $DIR!bash -c 'change_dir $DIR'!gtk-directory|Preferences!bash -c code_settings!gtk-preferences|Help!bash -c usage_info!gtk-info|Quit!bash -c quit_tray!gtk-quit" >&3
echo action:'bash -c "run_code '$DIR'"' >&3
yad --notification \
--image="$ICON" \
--auto-kill \
--no-middle \
--listen \
--text="Left-click to run selected code in terminal
Right-click for options menu" <&3

 
